//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#ifndef REC_MUTEX_TEST_H
#define REC_MUTEX_TEST_H

#include <TestBase.h>

class RecMutexTest : public TestBase
{
public:

    RecMutexTest();

private:

    virtual void run();
};

#endif
